@@ -1,5 +1,5 @@
 /* misc.c -  miscellaneous functions
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -24,11 +24,15 @@
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
+#ifdef HAVE_STAT
+#include <sys/stat.h>
+#endif
 #if defined(__linux__) && defined(__alpha__) && __GLIBC__ < 2
   #include <asm/sysinfo.h>
   #include <asm/unistd.h>
 #endif
 #ifdef HAVE_SETRLIMIT
+  #include <time.h>
   #include <sys/time.h>
   #include <sys/resource.h>
 #endif
@@ -235,7 +239,7 @@ print_cipher_algo_note( int algo )
 	if( !did_note ) {
 	    did_note = 1;
 	    log_info(_("this cipher algorithm is deprecated; "
-		       "please use a more standard one!x\n"));
+		       "please use a more standard one!\n"));
 	}
     }
 }
@@ -256,7 +260,7 @@ get_session_marker( size_t *rlen )
     static int initialized;
 
     if ( !initialized ) {
-        volatile ulong aa, bb; /* we really want the unitialized value */
+        volatile ulong aa, bb; /* we really want the uninitialized value */
         ulong a, b;
 
         initialized = 1;
@@ -275,5 +279,119 @@ get_session_marker( size_t *rlen )
     return marker;
 }
 
+/****************
+ * Wrapper around the libgcrypt function with addional checks on
+ * openPGP contraints for the algo ID.
+ */
+int
+openpgp_cipher_test_algo( int algo )
+{
+    if( algo < 0 || algo > 110 )
+        return G10ERR_CIPHER_ALGO;
+    return check_cipher_algo(algo);
+}
 
+int
+openpgp_pk_test_algo( int algo, unsigned int usage_flags )
+{
+    if( algo < 0 || algo > 110 )
+	return G10ERR_PUBKEY_ALGO;
+    return check_pubkey_algo2( algo, usage_flags );
+}
 
+int 
+openpgp_pk_algo_usage ( int algo )
+{
+    int use = 0; 
+    
+    /* they are hardwired in gpg 1.0 */
+    switch ( algo ) {    
+      case PUBKEY_ALGO_RSA:
+          use = PUBKEY_USAGE_SIG | PUBKEY_USAGE_ENC;
+          break;
+      case PUBKEY_ALGO_RSA_E:
+          use = PUBKEY_USAGE_ENC;
+          break;
+      case PUBKEY_ALGO_RSA_S:
+          use = PUBKEY_USAGE_SIG;
+          break;
+      case PUBKEY_ALGO_ELGAMAL_E:
+          use = PUBKEY_USAGE_ENC;
+          break;
+      case PUBKEY_ALGO_DSA:  
+          use = PUBKEY_USAGE_SIG;
+          break;
+      case PUBKEY_ALGO_ELGAMAL:
+          use = PUBKEY_USAGE_SIG | PUBKEY_USAGE_ENC;
+          break;
+      default:
+          break;
+    }
+    return use;
+}
+
+int
+openpgp_md_test_algo( int algo )
+{
+    if( algo < 0 || algo > 110 )
+        return G10ERR_DIGEST_ALGO;
+    return check_digest_algo(algo);
+}
+
+int
+check_permissions(const char *path,int checkonly)
+{
+#ifdef HAVE_STAT
+  struct stat statbuf;
+  int isdir=0;
+
+  if(opt.no_perm_warn)
+    return 0;
+
+  /* It's okay if the file doesn't exist */
+  if(stat(path,&statbuf)!=0)
+    return 0;
+
+  isdir=S_ISDIR(statbuf.st_mode);
+
+  /* The user doesn't own the file */
+  if(statbuf.st_uid != getuid())
+    {
+      if(!checkonly)
+	log_info(_("Warning: unsafe ownership on %s \"%s\"\n"),
+		 isdir?"directory":"file",path);
+      return 1;
+    }
+
+  /* This works for both directories and files - basically, we don't
+     care what the owner permissions are, so long as the group and
+     other permissions are 0. */
+  if((statbuf.st_mode & (S_IRWXG|S_IRWXO)) != 0)
+    {
+      char *dir;
+
+      /* However, if the directory the directory/file is in is owned
+         by the user and is 700, then this is not a problem.
+         Theoretically, we could walk this test up to the root
+         directory /, but for the sake of sanity, I'm stopping at one
+         level down. */
+
+      dir=make_dirname(path);
+      if(stat(dir,&statbuf)==0 && statbuf.st_uid==getuid() &&
+	 S_ISDIR(statbuf.st_mode) && (statbuf.st_mode & (S_IRWXG|S_IRWXO))==0)
+	{
+	  m_free(dir);
+	  return 0;
+	}
+
+      m_free(dir);
+
+      if(!checkonly)
+	log_info(_("Warning: unsafe permissions on %s \"%s\"\n"),
+		 isdir?"directory":"file",path);
+      return 1;
+    }
+#endif
+
+  return 0;
+}
