@@ -1,5 +1,5 @@
 /* status.c
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -30,6 +30,7 @@
     #include <sys/capability.h>
   #endif
   #ifdef HAVE_SYS_IPC_H
+    #include <sys/types.h>
     #include <sys/ipc.h>
   #endif
   #ifdef HAVE_SYS_SHM_H
@@ -136,6 +137,12 @@ get_status_string ( int no )
       case STATUS_POLICY_URL     : s = "POLICY_URL" ; break;
       case STATUS_BEGIN_STREAM   : s = "BEGIN_STREAM"; break;
       case STATUS_END_STREAM     : s = "END_STREAM"; break;
+      case STATUS_KEY_CREATED    : s = "KEY_CREATED"; break;
+      case STATUS_USERID_HINT    : s = "USERID_HINT"; break;
+      case STATUS_UNEXPECTED     : s = "UNEXPECTED"; break;
+      case STATUS_INV_RECP       : s = "INV_RECP"; break;
+      case STATUS_NO_RECP        : s = "NO_RECP"; break;
+      case STATUS_ALREADY_SIGNED : s = "ALREADY_SIGNED"; break;
       default: s = "?"; break;
     }
     return s;
@@ -193,7 +200,14 @@ write_status_text ( int no, const char *text)
     fputs ( get_status_string (no), statusfp );
     if( text ) {
         putc ( ' ', statusfp );
-        fputs ( text, statusfp );
+        for (; *text; text++) {
+            if (*text == '\n')
+                fputs ( "\\n", statusfp );
+            else if (*text == '\r')
+                fputs ( "\\r", statusfp );
+            else 
+                putc ( *(const byte *)text,  statusfp );
+        }
     }
     putc ('\n',statusfp);
     fflush (statusfp);
@@ -201,28 +215,43 @@ write_status_text ( int no, const char *text)
 
 
 /*
- * Write a status line with a buffer using %XX escapes.  
- * If WRAP is > 0 wrap the line after this length. 
+ * Write a status line with a buffer using %XX escapes.  If WRAP is >
+ * 0 wrap the line after this length.  If STRING is not NULL it will
+ * be prepended to the buffer, no escaping is done for string.
+ * A wrap of -1 forces spaces not to be encoded as %20.
  */
 void
-write_status_buffer ( int no, const char *buffer, size_t len, int wrap )
+write_status_text_and_buffer ( int no, const char *string,
+                               const char *buffer, size_t len, int wrap )
 {
     const char *s, *text;
-    int esc;
+    int esc, first;
+    int lower_limit = ' ';
     size_t n, count, dowrap;
 
     if( !statusfp )
 	return;  /* not enabled */
+    
+    if (wrap == -1) {
+        lower_limit--;
+        wrap = 0;
+    }
 
     text = get_status_string (no);
-    count = dowrap = 1;
+    count = dowrap = first = 1;
     do {
         if (dowrap) {
             fprintf (statusfp, "[GNUPG:] %s ", text );
             count = dowrap = 0;
+            if (first && string) {
+                fputs (string, statusfp);
+                count += strlen (string);
+            }
+            first = 0;
         }
         for (esc=0, s=buffer, n=len; n && !esc; s++, n-- ) {
-            if ( *s == '%' || *(const byte*)s <= ' ' ) 
+            if ( *s == '%' || *(const byte*)s <= lower_limit 
+                           || *(const byte*)s == 127 ) 
                 esc = 1;
             if ( wrap && ++count > wrap ) {
                 dowrap=1;
@@ -248,6 +277,13 @@ write_status_buffer ( int no, const char *buffer, size_t len, int wrap )
     fflush (statusfp);
 }
 
+void
+write_status_buffer ( int no, const char *buffer, size_t len, int wrap )
+{
+    write_status_text_and_buffer (no, NULL, buffer, len, wrap);
+}
+
+
 
 #ifdef USE_SHM_COPROCESSING
 
